/*
 *  Chesiresoft Calendar-Almanac
 *  (K) 1997, 1998, 1999 by Andrew Ziem.
 *
 *
 */

#ifndef __CDAY_H__
#define __CDAY_H__


/* -- compilers and #includes ---------------------------------------- */

/* GNU EMX/GCC v0.9c (OS/2) */
#if defined(__EMX__) && !defined(__RSXNT__)
#define COMPILER_NAME     "GNU EMX/GCC"
#define __OS2__
#define __32__
#define __FLAT__
#define FAR
#define INC_DIRPORT
#define INC_FNSPLIT
#define MAXDIR            255    /* this is bad for OS/2 but works for now */
#define MAXPATH           255
#define MAXDRIVE            5
#define MAXFILE            60
#define MAXEXT             15
#define strcmpi(a, b)     stricmp(a, b)
#define bzero(void, size)       memset(void, 0x00, size)
#define ffname(a)            (a).f.d_name
#endif

/* DJGPP v2.8 (DOS) */
#if defined(__DJGPP__)
#define COMPILER_NAME "DJGPP"
#define __DOS__
#define __32__
#define __FLAT__
#endif

/* Borland Borland C v5.0 */
#if defined(__BORLANDC__)
 #if defined(__WIN32__) == 1
 #include <dos.h>
 #define COMPILER_NAME     "Borland C v5"
 #define __32__
 #else
 #include <dos.h>
 #define COMPILER_NAME     "Borland C v5"
 #define __DOS__
 #define __16__
 #endif
#define bzero(void, size)    memset(void, 0x00, size)
#endif

/* Borland Turbo C v3.00 */
#if defined(__TURBOC__) && !defined(__BORLANDC__)
#include <dos.h>
#define COMPILER_NAME     "Borland Turbo C v3"
#define __DOS__
#define __16__
#define bzero(void, size)    memset(void, 0x00, size)
#endif

/* RSXNT v1.xx */
#if defined(__RSXNT__) 
#define COMPILER_NAME     "RSX"
#include <dos.h>
#include <sys/types.h>
#include <sys/dir.h>
#define __32__
#define WIN32
#define USE_FOPEN
#define INC_DIRPORT
#define INC_FNSPLIT
#define MAXPATH   255     /* this is really bad */
#define MAXDRIVE  3
#define MAXDIR    200
#define MAXFILE   200
#define MAXEXT    10      /* but it'll do for now */
#define bzero(void, size)          memset(void, 0x00, size)
#define ffname(a)         (a).file.cFileName
#endif

#if !defined(COMPILER_NAME)
#error no compatible compiler found
#endif


/* -- #includes ------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

/* EMX/GCC */
#if defined(INC_DIRPORT) && (defined(__EMX__) || defined(__RSXNT__))
#include "dirport.h"         /* SNIPPETS */
typedef DOSFileData          DIRSTRUCT;
#endif


#if !defined(INC_DIRPORT)
#include <dir.h>
typedef struct ffblk DIRSTRUCT;
#define  ffname(a)            (a).ff_name
#endif

#define FINDATTR            _A_NORMAL | _A_RDONLY | _A_ARCH

#if defined(INC_FNSPLIT)
#if defined(fnsplit)
 #undef fnsplit
 #endif
#if defined(fnmerge)
 #undef fnmerge
 #endif
#include "fnsplit.h"
#define fnsplit(a, b, c, d, e)     fnSplit(a, b, NULL, c, NULL, d, e)
#define fnmerge(a, b, c, d, e)     fnMerge(a, b, NULL, c, NULL, d, e)
#endif

#include "sniptype.h"

#if defined(INC_CALENDAR)
#include "lunconv.h"              /* glc */
#endif


/* -- some #definitions ---------------------------------------------- */

#define MAX_FGETS_READ       512

#define PROG_BASENAME        "Chesiresoft Calendar-Almanac"
#define PROG_BASEVER         "1.16"
#include "name.h"

#define PROG_DATE            __DATE__

#define FALSE 0   /* nothing is true, nothing is sacred */
#define TRUE 1    /* only one is true: nothing is true */

#define MALLOC(size,type)   (type *) malloc( (size) * sizeof( type ))

#define PAGE_WIDTH   78

#define ENGLISH

#define NUL '\0'


#define strip_trailing(a) if (a[strlen(a)-1]=='\n') a[strlen(a)-1]=NUL;

#define MY_EMAIL           "ziem@iex.net"
#define MY_WWW             "chesire.freeservers.com"

#define CONTACT "\nChesiresoft can be contacted via\n" \
                " E-mail: " MY_EMAIL "\n" \
                " WWW: " MY_WWW "\n"

#define ERR_MEMORY          -1


/* -- variables ------------------------------------------------------ */

typedef unsigned int bit;              /* shorthand for C bit fields */

typedef int TYPE;

typedef signed int YEAR;

typedef signed long JDN;  /* in 16DOS: 32bit: -2,147,483,648 to 2,147,483,647 */

typedef struct     /* command line options */
{
    bit sort            : 1;

    bit showmoon        : 1;  /* cmp */
    bit showastro       : 1;  /* not yet implemented */

    bit showdiscord     : 1;  /* cd */
    bit showfrench      : 1;  /* cf */
    bit showgoddesslunar: 1;  /* cl */
    bit showgue         : 1;  /* ce */
    bit showgregorian   : 1;  /* cg */

    bit showjusanotoron : 1;  /* cu */
    bit showjewish      : 1;  /* cje */
    bit showjulian      : 1;  /* cju */
    bit showilluminati  : 1;  /* ci */
    bit showmaya        : 1;  /* cma */
    bit showthelemic    : 1;  /* ct */

    bit showevents      : 1;  /* le */
    bit showbirthdays   : 1;  /* lb */
    bit showreminders   : 1;  /* lr */
} OPTS;

typedef struct
{
   int debug;
   int quiet;
} MOPTS;

/*
#define GREGORIAN       1
#define JULIAN          2
#define FRENCH          3
#define GODDESSLUNAR    4
#define THELEMIC        5
#define DISCORD         6
#define ILLUMANTI       7
#define MOON            8
*/

typedef struct
{
 unsigned int  mday,              /* day of month (1-31) */
               month;             /* month of year (1-12) */
 YEAR          year;              /* year (anything but 0) */
 unsigned int  yday;              /* day of year (1-366) */
 unsigned int  wday;              /* day of week (1-7, 1 = Sunday) */
 JDN           jdn;               /* full Julian Day Number wo/ time of day */
} GREG;

#define T_UNDEFINED 0
#define T_EVENT 1
#define T_REMINDER 2
#define T_BIRTHDAY 3

typedef struct
{
  YEAR             year;              /* zero means year unknown */
  TYPE             type;              /* T_... (above) */
  char             *text;
} DATAENTRY;

#define SIZEOF_DATAENTRY sizeof(DATAENTRY)

/* -- prototypes ----------------------------------------------------- */

extern void criterr(char const *errmsg,...);

#endif /* __CDAY_H__ */
