/*
 *  Chesiresoft Calendar-Almanac
 *  (K) 1997, 1998, 1999 by Andrew Ziem <ziem@iex.net>
 *  http://chesire.freeservers.com
 *
 *     FIGURE.C -- figurin' routines
 *
 *
 *  Arguments used in most functions:
 *    daynumber = Gregorian day of year (1-366)
 *    year      = Gregorian year
 *
 *
 *  Warning:
 *    - not much checking is done to see if passed values are valid!
 *
 */


#include "figure.h"



/*
   ** Return: year's Golden Number (1-19).
   ** Warning: untested.
   **
 */

#define golden_number(year)      ((year % 19) + 1)


/*
   ** Return: year's Epact (1-30), Gregorian method
   ** Accepts: positive years
   ** Warning: untested
   **
   ** "In the Gregorian calendar, the Epact is the age of the moon at the
   ** start of the year."
   **
   ** Adapted to C from the Calendar FAQ.
   **
 */

int
  epact(YEAR year)
{
    unsigned int epact = 0, century;

    century = (year / 100) + 1;

    epact = (11 * (golden_number(year) - 1)) % 30;
    epact = epact - (3 * century) / 4;
    epact = epact + (8 * century + 5) / 25;
    epact = epact + 8;
/* 5) Add or subtract 30 until the Epact lies between 1 and 30. */
    if (epact > 30)
	  epact -= 30;

    return epact;

}					  /* epact() */


/*
   ** Return: phase of the moon for a certian point in time (0-7, 0=new).
   **
   ** Stolen from Doorskel, where it was stolel from Lynn Nash, who hacked it
   ** from BSD Unix (I think).  Tweeked by myself.
   **
 */

unsigned int
  Moon_Phase(YEAR year, int diy)
{
    int phase;			  /* Moon phase                  */


    /* Calculate the phase, using the magic numbers defined above. Note that
     * (phase and 7) is equivalent to (phase mod 8) and is needed on two days
     * per year (when the algorithm yields 8). */

    phase = (((((diy + epact(year)) * 6) + 11) % 177) / 22) & 7;

    if (8 == phase)
    {
	  printf("debug: phase 8!!!\n");
	  phase = (((((diy + epact(year)) * 6) + 11) % 177) / 22) % 8;
    }

    return phase;
}


/*
   ** Return: is date valid in the Gregorian calendar?  (0 = false, 1 = true)
   ** From: DAYNUM.C by Bob Stout.
   **
   ** Modified to not accept the year '0' and slightly optimized.
   **
 */

int
  checkdate(unsigned yr, unsigned mo, unsigned day)
{
    unsigned int days[] =
    {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

    if ((1 > mo || 12 < mo) ||
	  (1 > day || day > (days[mo - 1] + (2 == mo && isleap(yr)))) ||
	  (0 == yr))
	  return FALSE;
    else
	  return TRUE;

}


/*
 ** Return: day of week in French Republican calendar (1..10)
 ** By: Andrzej Ziem
 **
 */

int french_dow(int mday)
{
    if (0 == mday % 10)
          return (int) (10);
		else
	  return (int) (mday % 10);
}



