/*
 *  Chesiresoft Calendar-Almanac
 *  (K) 1997, 1998, 1999 by Andrew Ziem <ziem@iex.net>
 *  http://chesire.freeservers.com
 *
 *     LIBRARY.C -- library access routines
 *
 *
 *  Problems and Concerns:
 *    - handle leap day
 *    - Day of Week (DoW) mishandled
 *    - '!' mishandled (not handled)
 *
 *
 */


#include "library.h"



int CompChart(const void *C1, const void *C2)
{
    int RetVal;


    if (0 != ((DATAENTRY *) C1)->year)
    {
	  RetVal = ((DATAENTRY *) C2)->year -
		((DATAENTRY *) C1)->year;
    }

    if (0 == RetVal)
	  RetVal = stricmp(((DATAENTRY *) C2)->text, ((DATAENTRY *) C1)->text);

    CompareResult = RetVal;

    /*  Return a 'match' (0) if the item in the list is 'larger' than the
       new data.
     */
    return RetVal >= 0 ? 0 : 1;
}					  /* CompChart() */

int sort_data(void)
{
    DATAENTRY d;
    int ListSort;


    ListSort = LLScreate(SIZEOF_DATAENTRY);

    if (ERR_MEMORY == ListSort)
    {
	  fprintf(stderr, "\n" HAIKU_MEM "\n");
	  fprintf(stderr, "warning: unable to initilize linked list for sorting (memory error)\n");
	  return FALSE;
    }

    if (!LLSnodePtr2First(ListEvents))
    {
	  fprintf(stderr, "warning: unable to initilize linked list for sorting\n");
	  return FALSE;
    }

    do
    {

	  LLSnodeDataTo(ListEvents, &d);
	  LLSnodePtr2First(ListSort);
	  LLSnodeFind(ListSort, CompChart, &d);
	  if (0 <= CompareResult)
		LLSnodeInsertFrom(ListSort, &d);
	  else
		LLSnodeAddFrom(ListSort, &d);
//      LLSnodeDelete(ListEvents);

    }
    while (LLSnodePtr2Next(ListEvents));

    LLSdelete(ListEvents);
    ListEvents = ListSort;

    return TRUE;

}					  /* sort_data() */


/* pre search library function, find appropriate libraries */
void add_library(char *fname)
{
    DIRSTRUCT dirfile;
    char s[MAXPATH], s2[MAXPATH], ss[6];
    char drive[MAXDRIVE], dir[MAXDIR], file[MAXFILE], ext[MAXEXT];
    int done;


    fnsplit(fname, drive, dir, file, ext);


    if (0 == strlen(file) || '.' == file[0])
	  strcpy(file, "*");
    if (0 == strlen(ext))
	  strcpy(ext, ".*");

    fnmerge(s, drive, dir, file, ext);

    done = FIND_FIRST(s, FINDATTR, &dirfile);

    while (!done)
    {
	  char ext2[MAXEXT], file2[MAXFILE];
	  int use;


	  fnsplit(ffname(dirfile), NULL, NULL, file2, ext2);

	  use = (0 == stricmp(ext2, ".ALL"));
	  if (!use)
		use = (0 == stricmp(ext2, ".OWN"));
	  if (!use)
	  {
		strcpy(ss, ".");
		strcat(ss, gregorian_months[greg.month - 1]);
		ss[4] = '\0';
		use = (0 == stricmp(ext2, ss));
	  }

	  if (use)
	  {
		fnmerge(s, drive, dir, file2, ext2);
		search_datafile(s);
	  }

	  done = FIND_NEXT(&dirfile);
    }

#if defined(__OS2__)
    FIND_END(&dirfile);
#endif


}					  /* add_library() */



int add_entry(char *sinfo1, char *sinfo2, char savebit)
{
    DATAENTRY dataentry;
    char t, sb[2];
    char smon[4], sday[4], syear[6];


    /* clear */
    bzero(&dataentry, SIZEOF_DATAENTRY);

    t = sinfo1[0];

    /* fill */
    if (opts.showbirthdays && t == 'B')
	  dataentry.type = T_BIRTHDAY;
    else if (opts.showevents && t == 'S')
	  dataentry.type = T_EVENT;
    else
	  return FALSE;		  /* bad type */


    sinfo1++;
    strncpy(smon, sinfo1, 2);
    sinfo1 = sinfo1 + 2;
    strncpy(sday, sinfo1, 2);
    sinfo1 = sinfo1 + 2;
    strncpy(syear, sinfo1, 4);
    sinfo1 = sinfo1 - 4;

    sb[0] = savebit;
    sb[1] = NUL;

    if (atoi(sb) && atoi(sb) != greg.wday)
	  return FALSE;

    if (atoi(smon) != greg.month || atoi(sday) != greg.mday)
	  return FALSE;

#if defined(__OS2__)
    if (mopts.debug)
    {
	  printf("debug: atoi(smon) = %i\n", atoi(smon));
	  printf("debug: atoi(syear, raw-premodified) = %i\n", atoi(syear));
    }
#endif

    dataentry.year = atoi(syear);

#if defined(__EMX__) && !defined(__RSXNT__)
    if (dataentry.year > 10000)
	  dataentry.year = dataentry.year / 10;
#endif


    /* continued */
    if ('C' == savebit)
    {
	  DATAENTRY d2;


	  LLSnodePtr2Last(ListEvents);
	  LLSnodeDataTo(ListEvents, &d2);

	  dataentry.text = MALLOC(strlen(sinfo2) + strlen(d2.text) + 2, char);

	  if (NULL == dataentry.text)
	  {
		fprintf(stderr, "error: memory full; unable to allocate %i bytes\n", strlen(sinfo2) + 1);
		return FALSE;
	  }

	  strcpy(dataentry.text, d2.text);
	  strcat(dataentry.text, sinfo2);

	  free(d2.text);
	  LLSnodeDelete(ListEvents);

	  LLSnodeAppendFrom(ListEvents, &dataentry);

    }
    else
    {
	  /* original */

	  dataentry.text = MALLOC(strlen(sinfo2) + 1, char);

	  if (NULL == dataentry.text)
	  {
		fprintf(stderr, "error: memory full; unable to allocate %i bytes\n", strlen(sinfo2) + 1);
		return FALSE;
	  }

	  strcpy(dataentry.text, sinfo2);

	  LLSnodeAppendFrom(ListEvents, &dataentry);
	  events++;
    }

    return TRUE;

}					  /* add_entry() */


void search_datafile(char *fname)
{
    FILE *in;
    char line[MAX_FGETS_READ], *pstr, *pstr2, savebit;


    if (mopts.debug)
    {
	  printf("debug: searching database %s...\n", fname);
	  printf("debug2: raw-date(search) %i-%i-%i\n", greg.year, greg.month, greg.mday);
    }

    in = fopen(fname, "rt");

    if (!in)
    {
       printf("unable to open %s for reading\n", fname);
       return;
    }

    while (NULL != fgets(line, MAX_FGETS_READ, in))
    {
	  if (strlen(line) > 12 && '*' != line[0])
	  {
		pstr = line;
		/* 01234567890 */
		/* B03011904 Glenn Miller */
		savebit = pstr[9];
		pstr[9] = NUL;
		pstr2 = &pstr[10];

		switch (pstr[0])
		{
		    case 'B':
		    case 'S':

//              printf("debug: '%c', '%s'\n", savebit, pstr2);
			  add_entry(pstr, pstr2, savebit);
			  break;

		}
	  }
    }
    fclose(in);

}					  /* search_datafile() */


/* probably very inefficent, but it does work */
void show_entry(char *string, int indent)
{
    char *line, *end_line;
    size_t spaces;
    int last = 0, first = 0;
    size_t len;


    word_wrap(string, PAGE_WIDTH - indent);
    line = string;

    while (!last)
    {
	  end_line = strchr(line, '\n');

	  if (NULL == end_line)
	  {
		last = 1;
		end_line = strchr(line, '\0');
	  }

	  len = end_line - line;

	  if (0 != first)
	  {
		printf("\n");
		if (indent)
		    printf("%*c", indent, ' ');
	  }
	  else
		first++;

	  printf("%*.*s", len, len, line);

	  line = end_line + 1;
    }
}					  /* show_entry() */


void
  show_data(TYPE showtype)
{
    int first = 0;
    DATAENTRY dataentry;


    if (!LLSnodePtr2First(ListEvents))
	  return;

    do
    {

	  LLSnodeDataTo(ListEvents, &dataentry);

	  if (dataentry.type == showtype)
	  {
		if (0 == first)
		{
		    switch (showtype)
		    {
			  case T_EVENT:

				printf("\nOn this day...\n");
				break;

			  case T_BIRTHDAY:

				printf("\nHappy Birthday to...\n");
				break;

			  case T_REMINDER:

				printf("\nDon't forget...\n");
				break;

		    }

		    ++first;

		}

		if (0 != dataentry.year)
		    printf("In %4i ", dataentry.year);

		show_entry(dataentry.text, dataentry.year == 0 ? 2 : 8);
		printf("\n");
	  }
    }
    while (LLSnodePtr2Next(ListEvents));

}					  /* show_data() */
